/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2016 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "fixedFluxMassCFvPatchScalarField.H"
#include "fvPatchFieldMapper.H"
#include "volFields.H"
#include "surfaceFields.H"
#include "addToRunTimeSelectionTable.H"
#include "physicoChemicalConstants.H"

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::fixedFluxMassCFvPatchScalarField::fixedFluxMassCFvPatchScalarField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF
)
:
    fixedGradientFvPatchScalarField(p, iF),
    currentDensity_(0.0),
    molarMassA_(0.0),
    valencyNumberOfIons_(0.0),
    massDiffusivityAB_(0.0)
{}


Foam::fixedFluxMassCFvPatchScalarField::fixedFluxMassCFvPatchScalarField
(
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const dictionary& dict
)
:
    fixedGradientFvPatchScalarField(p, iF),
    currentDensity_(readScalar(dict.lookup("j"))),
    molarMassA_(readScalar(dict.lookup("M_A"))),
    valencyNumberOfIons_(readScalar(dict.lookup("n_el"))),
    massDiffusivityAB_(readScalar(dict.lookup("D_AB")))
{
    if (dict.found("value") && dict.found("gradient"))
    {
        fvPatchField<scalar>::operator=
        (
            scalarField("value", dict, p.size())
        );
        gradient() = scalarField("gradient", dict, p.size());
    }
    else
    {
        fvPatchField<scalar>::operator=(patchInternalField());
        gradient() = 0.0;
    }
}


Foam::fixedFluxMassCFvPatchScalarField::fixedFluxMassCFvPatchScalarField
(
    const fixedFluxMassCFvPatchScalarField& ptf,
    const fvPatch& p,
    const DimensionedField<scalar, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    fixedGradientFvPatchScalarField(p, iF),
    currentDensity_(ptf.currentDensity_),
    molarMassA_(ptf.molarMassA_),
    valencyNumberOfIons_(ptf.valencyNumberOfIons_),
    massDiffusivityAB_(ptf.massDiffusivityAB_)
{
    patchType() = ptf.patchType();

    // Map gradient. Set unmapped values and overwrite with mapped ptf
    gradient() = 0.0;
    gradient().map(ptf.gradient(), mapper);

    // Evaluate the value field from the gradient if the internal field is valid
    if (notNull(iF) && iF.size())
    {
        scalarField::operator=
        (
            patchInternalField() + gradient()*(patch().nf() & patch().delta())
        );
    }
    else
    {
        this->map(ptf, mapper);
    }
}


Foam::fixedFluxMassCFvPatchScalarField::fixedFluxMassCFvPatchScalarField
(
    const fixedFluxMassCFvPatchScalarField& wbppsf
)
:
    fixedGradientFvPatchScalarField(wbppsf),
    currentDensity_(wbppsf.currentDensity_),
    molarMassA_(wbppsf.molarMassA_),
    valencyNumberOfIons_(wbppsf.valencyNumberOfIons_),
    massDiffusivityAB_(wbppsf.massDiffusivityAB_)
{}


Foam::fixedFluxMassCFvPatchScalarField::fixedFluxMassCFvPatchScalarField
(
    const fixedFluxMassCFvPatchScalarField& wbppsf,
    const DimensionedField<scalar, volMesh>& iF
)
:
    fixedGradientFvPatchScalarField(wbppsf, iF),
    currentDensity_(wbppsf.currentDensity_),
    molarMassA_(wbppsf.molarMassA_),
    valencyNumberOfIons_(wbppsf.valencyNumberOfIons_),
    massDiffusivityAB_(wbppsf.massDiffusivityAB_)
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::fixedFluxMassCFvPatchScalarField::updateCoeffs()
{
    if (updated())
    {
        return;
    }
    gradient() = (currentDensity_*molarMassA_)/(constant::physicoChemical::F.value()*valencyNumberOfIons_*massDiffusivityAB_);
    fixedGradientFvPatchScalarField::updateCoeffs();
}


void Foam::fixedFluxMassCFvPatchScalarField::write(Ostream& os) const
{
    fixedGradientFvPatchScalarField::write(os);
    os.writeKeyword("j") << currentDensity_ << token::END_STATEMENT << nl;
    os.writeKeyword("M_A") << molarMassA_ << token::END_STATEMENT << nl;
    os.writeKeyword("n_el") << valencyNumberOfIons_ << token::END_STATEMENT << nl;
    os.writeKeyword("D_AB") << massDiffusivityAB_ << token::END_STATEMENT << nl;
    writeEntry("value", os);
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{
    makePatchTypeField
    (
        fvPatchScalarField,
        fixedFluxMassCFvPatchScalarField
    );
}


// ************************************************************************* //
