/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2011-2016 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Application
    pimpleBoussinesqHMctFoam

Description
    Transient solver for buoyant, turbulent flow of incompressible fluids.

    Uses the Boussinesq approximation:
    \f[
        rho_{k} = 1 - beta_T (T - T_{ref}) - beta_rho_A (rho_A - rho_A_{ref})
    \f]

    where:
        \f$ rho_{k} \f$ = the effective (driving) kinematic density
        beta_T = thermal expansion coefficient [1/K]
        beta_rho_A = compositional expansion coefficient [1/1]
        T = temperature [K]
        rho_A = mass concentration of A [kg/m^3]
        \f$ T_{ref} \f$ = reference temperature [K]

    Valid when:
    \f[
        \frac{beta_T(T - T_{ref})}{rho_{ref}} << 1
        \frac{beta_rho_A(rho_A - rho_A_{ref})}{rho_{ref}} << 1
    \f]

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "singlePhaseTransportModel.H"
#include "turbulentTransportModel.H"
#include "fvOptions.H"
#include "pimpleControl.H"
#include "deflatedPCG.H"
#include "clusterControl.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{
    #include "postProcess.H"
    #include "setRootCase.H"
    #include "gitVersion.H"
    #include "createTime.H"
    #include "createMesh.H"
    #include "createControl.H"
    #include "createSolverParameters.H"
    #include "createFvOptions.H"
    #include "createFields.H"
    #include "CourantNo.H"
    #include "createTimeControls.H"
    #include "setInitialDeltaT.H"

    #include "initContinuityErrs.H"


    turbulence->validate();

    // * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    const bool massConcentration = runTime.controlDict().lookupOrDefault<bool>(
    "massConcentration", false
    );
    const bool thermal = runTime.controlDict().lookupOrDefault<bool>(
    "thermal", false
    );

    Info<< "\nStarting time loop\n" << endl;
    while (runTime.run())
    {
        #include "readTimeControls.H"
        #include "CourantNo.H"
        #include "setDeltaT.H"
        #include "continuityErrs.H"
        runTime++;
        Info<< "Time = " << runTime.timeName() << nl << endl;
        // --- Pressure-velocity PIMPLE corrector loop
        while (pimple.loop())
        {
            #include "UEqn.H"
            rhok = 1.0;
            if(massConcentration){
            #include "massEqn.H"
            }
            if(thermal){
            #include "TEqn.H"
            }
            // --- Pressure corrector loop
            while (pimple.correct())
            {
                #include "pEqn.H"
            }
            if (pimple.turbCorr())
            {
                laminarTransport.correct();
                turbulence->correct();
            }
        }
        #include "computeFields.H"
        #include "write.H"
    }

    Info<< "End\n" << endl;

    return 0;
}


// ************************************************************************* //
