/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     | Website:  https://openfoam.org
    \\  /    A nd           | Copyright (C) 2019 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::spaceVariableFluxMassFvPatchScalarField

Description
    This boundary condition provides a spaceVariableFluxMass condition,
    calculated as:

        \f[
            \dot{m}'' (x, y) = ((j_0 * M_A)/(F * D_AB)) *  g(x, y)
        \f]

    where
    \vartable
        j_0    | scalar   | reference current density [A m^{-2}]
        M_A    | scalar   | molar mass of the component A [kg mol^{-1}]
        F      | scalar   | Faraday constant [] | constant::physicoChemical::F.value()
        D_AB   | scalar   | Diffusion  [m^{2}/s]
        g(x, y)| function | distribution of the current in the two dimensions
    \endtable

Usage
    \table
        Property   | Description                  | Req'd? | Default
        scalarData | single scalar value          | yes    |
        data       | single scalar value          | yes    |
        fieldData  | scalar field across patch    | yes    |
        timeVsData | scalar function of time      | yes    |
        wordData   | word, eg name of data object | no     | wordDefault
    \endtable

    Example of the boundary condition specification:
    \verbatim
    <patchName>
    {
        type       spaceVariableFluxMass;
        meanCurrentDensity 1;
        scalarData -1;
        data       1;
        fieldData  uniform 3;
        timeVsData table (
                             (0 0)
                             (1 2)
                          );
        wordName   anotherName;
        value      uniform 4;
    }
    \endverbatim

SourceFiles
    spaceVariableFluxMassFvPatchScalarField.C

\*---------------------------------------------------------------------------*/

#ifndef spaceVariableFluxMassFvPatchScalarField_H
#define spaceVariableFluxMassFvPatchScalarField_H

#include "fvPatchFields.H"
#include "fixedGradientFvPatchFields.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
     Class spaceVariableFluxMassFvPatchScalarField Declaration
\*---------------------------------------------------------------------------*/

class spaceVariableFluxMassFvPatchScalarField
:
    public fixedGradientFvPatchScalarField
{
    // Private data

        //- Single valued scalar quantity, e.g. a coefficient
        scalar meanCurrentDensity_;

        //- Single valued Type quantity, e.g. reference pressure pRefValue_
        //  Other options include vector, tensor
        scalar value0_;
        scalar value1_;
        scalar value2_;
        scalar value3_;
        scalar value4_;
        scalar value5_;
        //- Field of Types, typically defined across patch faces
        //  e.g. total pressure p0_.  Other options include vectorField
        // scalarField fieldCurrentDensity_; it can be used

        //- Boolean for true/false, e.g. specify if flow rate is volumetric_
        // bool boolData_;
        //- Direction of the x-xcoordinate
        vector x_;
public:
    //- Runtime type information
    TypeName("spaceVariableFluxMassC");


    // Constructors

        //- Construct from patch and internal field
        spaceVariableFluxMassFvPatchScalarField
        (
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&
        );

        //- Construct from patch, internal field and dictionary
        spaceVariableFluxMassFvPatchScalarField
        (
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&,
            const dictionary&
        );

        //- Construct by mapping given mixedTypeFvPatchField
        //  onto a new patch
        spaceVariableFluxMassFvPatchScalarField
        (
            const spaceVariableFluxMassFvPatchScalarField&,
            const fvPatch&,
            const DimensionedField<scalar, volMesh>&,
            const fvPatchFieldMapper&
        );

        //- Construct as copy
        spaceVariableFluxMassFvPatchScalarField
        (
            const spaceVariableFluxMassFvPatchScalarField&
        );

        //- Construct and return a clone
        virtual tmp<fvPatchScalarField> clone() const
        {
            return tmp<fvPatchScalarField>
            (
                new spaceVariableFluxMassFvPatchScalarField(*this)
            );
        }

        //- Construct as copy setting internal field reference
        spaceVariableFluxMassFvPatchScalarField
        (
            const spaceVariableFluxMassFvPatchScalarField&,
            const DimensionedField<scalar, volMesh>&
        );

        //- Construct and return a clone setting internal field reference
        virtual tmp<fvPatchScalarField> clone
        (
            const DimensionedField<scalar, volMesh>& iF
        ) const
        {
            return tmp<fvPatchScalarField>
            (
                new spaceVariableFluxMassFvPatchScalarField(*this, iF)
            );
        }


    // Member functions

        // Mapping functions
            // mean current density
            scalar& meanCurrentDensity()
            {
                return meanCurrentDensity_;
            }

            scalar& value0()
            {
                return value0_;
            }
            scalar& value1()
            {
                return value1_;
            }
            scalar& value2()
            {
                return value2_;
            }
            scalar& value3()
            {
                return value3_;
            }
            scalar& value4()
            {
                return value4_;
            }
            scalar& value5()
            {
                return value5_;
            }
        //- Return x direction
            vector& x()
            {
                return x_;
            }
        // Evaluation functions

            //- Update the coefficients associated with the patch field
            virtual void updateCoeffs();


        //- Write
        virtual void write(Ostream&) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
#include "volFields.H"

#endif

// ************************************************************************* //
